from django.conf import settings as django_settings
from inertia.settings import settings as inertia_settings
from pathlib import Path

BASE_DIR = getattr(django_settings, "BASE_DIR")

MIDDLEWARES = [
    "inertia.middleware.InertiaMiddleware",
    # "django-breeze.django_breeze.middleware.inertia_share",
]

INSTALLED_APPS = [
    'inertia',
    'django_vite',
]

# Django Breeze Default Settings
DJANGO_BREEZE: dict = {
    "INERTIA": {
        "LAYOUT": "index.html",
        "SSR_URL": inertia_settings.INERTIA_SSR_URL,
        "SSR_ENABLED": inertia_settings.INERTIA_SSR_ENABLED,
        "JSON_ENCODER": inertia_settings.INERTIA_JSON_ENCODER,
    },
    "DJANGO_VITE": {
        "DEV_MODE": getattr(django_settings, "DEBUG", True),
        "SERVER_PROTOCOL": "http",
        "DEV_SERVER_HOST": "localhost",
        "DEV_SERVER_PORT": 5173,
        "WS_CLIENT_URL": "@vite/client",
        "ASSETS_PATH": Path(getattr(django_settings, "STATIC_PATH", 'static')) / "dist",
        "STATIC_URL_PREFIX": "",
        "LEGACY_POLYFILLS_MOTIF": "legacy-polyfills",
    },
    "STATIC_ROOT": "static",
    "CSRF_HEADER_NAME": "HTTP_X_XSRF_TOKEN",
    "CSRF_COOKIE_NAME": "XSRF-TOKEN",
}


# Merge DJANGO_BREEZE and user-defined settings
user_settings = getattr(django_settings, "DJANGO_BREEZE", {})

def merge_dicts(*dicts):
    """
    Recursively merges dictionaries.
    """
    result = {}
    for d in dicts:
        for k, v in d.items():
            if isinstance(v, dict):
                result[k] = merge_dicts(result.get(k, {}), v)
            else:
                result[k] = v
    return result

merged_settings = merge_dicts(DJANGO_BREEZE, user_settings)


def key_exist(key) -> bool:
    return hasattr(django_settings, key) and getattr(django_settings, key) is not None

settings = {}

for key, value in merged_settings.items():
    if isinstance(value, dict):
        for sub_key, sub_value in value.items():
            sub_key = f"{key}_{sub_key}"
            if not key_exist(sub_key):
                settings[sub_key] = sub_value
    else:
        if not key_exist(key):
            settings[key] = value
            
for key, value in settings.items():
    setattr(django_settings, key, value)
    
django_settings.TEMPLATES[0]["DIRS"].append(Path(BASE_DIR) / "src/")

for app in INSTALLED_APPS:
    django_settings.INSTALLED_APPS.append(app)
    
for middleware in MIDDLEWARES:
    django_settings.MIDDLEWARE.append(middleware)
# django_settings.STATICFILES_DIRS.append(settings.DJANGO_VITE_ASSETS_PATH)



import argparse
import shutil
import os
import glob
import fileinput
import subprocess
from pathlib import Path, PurePath
from django.conf import settings


BASE_DIR = Path(__file__).resolve().parent.parent.parent


class Manager:
    def __init__(self) -> None:
        # project_name = os.environ.get('DJANGO_SETTINGS_MODULE')
        # project_name = getattr(settings, 'ROOT_URLCONF')
        # DJANGO_BASE_DIR = settings.BASE_DIR
        # import re
        # project_name = re.sub(r'\.urls$', '', settings.ROOT_URLCONF)
        settings_dir = os.path.dirname(os.path.abspath(__file__))
        project_name = os.path.basename(settings_dir)
        print(project_name)

    def args_parser(self):
        parser_des = """\
        Copy React or TypeScript files from templates 
        folder to current working directory.
        """
        parser = argparse.ArgumentParser(description=parser_des)
        parser.add_argument("project_name", type=str, help="Name of the django project")
        parser.add_argument(
            "framework", choices=["react", "vue"], help="The framework to use"
        )
        parser.add_argument(
            "--typescript", action="store_true", help="Include if using TypeScript"
        )
        parser.add_argument(
            "--fresh",
            action="store_true",
            help="If the existing destination directory should be overide.",
        )
        parser.add_argument(
            "--verbose",
            action="store_true",
            help="Display progress information",
            default=True,
        )

        self.args = parser.parse_args()
        return self.args

    def execute_command(self):
        args = self.args_parser()

        self.PROJECT_BASE_DIR = os.getcwd()

        # check django project name and existence settings
        self.PROJECT_DIR = Path(args.project_name)
        self.PROJECT_DIR = Path(args.project_name)
        self.PROJECT_DIR = Path(args.project_name)

        print((self.PROJECT_BASE_DIR))
        print((self.PROJECT_DIR))
        # copy neccessary project files
        self.copy_files()
        # self.configs()

    def copy_files(self):
        # install packages
        # self.install_packages()

        TEMPLATE_DIR = BASE_DIR / "templates"
        DESTINATION_DIR = os.getcwd()

        if self.args.framework == "react":
            src_dir = TEMPLATE_DIR / "react"
        elif self.args.framework == "vue":
            src_dir = TEMPLATE_DIR / "vue"
        else:
            print("You must provide framework to use! suported frameworks [react, vue]")

        if self.args.typescript:
            src_dir += "_typescript"

        # if not os.path.exists(DESTINATION_DIR):
        #     os.makedirs(DESTINATION_DIR)
        try:
            shutil.copytree(src_dir, DESTINATION_DIR, dirs_exist_ok=self.args.fresh)
            if self.args.verbose:
                self._verbose(
                    f"Files created successfully. Source Directory is: {DESTINATION_DIR}\n"
                )
        except FileExistsError as e:
            print(
                f"Directory '{DESTINATION_DIR}' already exist! Use --fresh to overide the existing folder"
            )
            return

    def configs(self):
        # Modify settings variables
        self._verbose("Configuring project settings...")
        settings.INERTIA_LAYOUT = "index.html"
        settings.CSRF_HEADER_NAME = "HTTP_X_XSRF_TOKEN"
        settings.CSRF_COOKIE_NAME = "XSRF-TOKEN"

        if "django_vite" not in settings.INSTALLED_APPS:
            settings.INSTALLED_APPS += ["inertia", "django_vite"]
        if "inertia" not in settings.INSTALLED_APPS:
            settings.INSTALLED_APPS += ["inertia"]

        self._verbose("Settings modified successfully!\n")

    def project_configs(self):
        settings_files = glob.glob("**/settings.py", recursive=True)
        if settings_files:
            settings_file = settings_files[0]
            print(f"Found settings file: {settings_file}")

            # Add installed apps
            settings_file.INSTALLED_APPS.append("myAPp")

            # Add variables to settings.py file
            variables = {
                "STATIC_URL": "static/",
                "INERTIA_LAYOUT": "index.html",
                "CSRF_HEADER_NAME": "HTTP_X_XSRF_TOKEN",
                "CSRF_COOKIE_NAME": "XSRF-TOKEN",
                "DJANGO_VITE_ASSETS_PATH": 'BASE_DIR / "static" / "dist"',
                "DJANGO_VITE_DEV_MODE": "DEBUG",
            }

            added_variables = {key: False for key in variables}
            for line in fileinput.input(settings_file, inplace=True):
                for key, value in variables.items():
                    if line.startswith(key):
                        print(f'{key} = "{value}"')
                        added_variables[key] = True
                        break
                else:
                    print(line.rstrip())
            # Add variables that were not found in the file
            for key, value in variables.items():
                if not added_variables[key]:
                    with open(settings_file, "a") as f:
                        f.write(f'\n{key} = "{value}"\n')
        else:
            print("Could not find settings file in base directory.")

    def install_packages(
        self,
    ):
        """Install required Pypi packages for this project"""
        # Install pip packages
        packages = ["django-vite", "django-inertia"]
        print("\nInstalling required packages please wait...")
        for package in packages:
            subprocess.run(["pip", "install", package], stdout=subprocess.PIPE)
            self._verbose(f"Installed package {package}")
        print("Packages installed successfully!\n")

    def _verbose(self, message: str):
        """Display brief message of a process(es)

        Args:
            message (str): message to display
        """
        if self.args.verbose:
            print(message)



#!/usr/bin/env python
"""Django's command-line utility for administrative tasks."""
import os
import sys


def main():
    # """Run administrative tasks."""

    # try:
    from core.management import Manager

    # except ImportError as exc:
    #     raise ImportError(
    #         "Couldn't import Django. Are you sure it's installed and "
    #         "available on your PYTHONPATH environment variable? Did you "
    #         "forget to activate a virtual environment?"
    #     ) from exc
    manager = Manager()
    manager.execute_command()


if __name__ == "__main__":
    main()
